@@ -42,6 +42,8 @@ def hermite_nf(M: Matrix) -> Matrix:
             H[:, jj] -= tmp * H[:, j]
         #print(H)
         j -= 1
+        if j < 0:
+            break
     while H.cols> 1 and all(not H[i, 0] for i in range(m)): # remove zero columns
         H = H[:,1:]
     return H
@@ -79,7 +81,7 @@ def babai_round_cvp(x: Matrix, U: Matrix) -> Matrix:
     return U * k
 
 def babai_round_bnd(U: Matrix) -> float:
-    "Bound for Babai's rounding algorithm for solving the CVP."
+    "Bound for Babai's rounding algorithm for solving the CVPwith respect to the sup norm."
     return floor(1 / (2 * max([ U.inv()[i,:].norm(1) for i in range(U.rows)])))
 
 def babai_plane_cvp(x: Matrix, U: Matrix) -> Matrix:
@@ -91,7 +93,7 @@ def babai_plane_cvp(x: Matrix, U: Matrix) -> Matrix:
     return (x - y).applyfunc(round)
 
 def babai_plane_bnd(U: Matrix, p = 2) -> float:
-    "Bound for Babai's closest plane algorithm for solving the CVP."
+    "Bound for Babai's closest plane algorithm for solving the CVP with respect to the Euclidean norm (p=2) or sup norm (p=inf)."
     Us = gram_schmidt(U)[0]
     return float(0.5 * min([Us[:, i].norm(p) for i in range(Us.rows)]))
 
@@ -173,7 +175,7 @@ def lll(V: Matrix, delta: float = 0.75, sort: bool = True) -> Matrix:
     return U
 
 def random_unimodular_matrix(n: int, iterations: int = 50, max_val: int = None) -> Matrix:
-    "Create a random unimodular matrix of dimension n."
+    "Create a pseudorandom unimodular matrix of dimension n."
     W = zeros(n)
     for i in range(n):
         for j in range(i, n):
